/*
 * Created on Aug 30, 2004
 *
 * Connect to the HSqlDb database
 */
package nl.amis.workshop;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author zeger
 *
 * Connect to the HSqlDb database
 */
public class DbConnect 
{
    private static Log log = LogFactory.getLog(DbConnect.class);
    // these constants should be in our .properties file...
    // Preferably in a safe and monitored directory....
    private static final String driver = "org.hsqldb.jdbcDriver";
    // This URL implies an in-memory database!
    private static final String URL = "jdbc:hsqldb:.";
    private static final String user = "sa";
    private static final String passwd = "";

    private static Connection connection = null;
	
    /**
     * Init the db connection, if it isn't already done
     * Create the person table as well (since we're in memory).
     */
    private static void createConnection()
    {
        // LOAD AND REGISTER THE JDBC DRIVER
        try
        {
            // only create a connection if there isn't any      
            if (connection == null || connection.isClosed())
            {
                log.trace("Loading JDBC Driver -> " + driver);
                Class.forName (driver).newInstance();
                
            	log.debug("Trying to create a connection...");
                connection = 
                	DriverManager.getConnection(URL, user, passwd);
                log.debug("Connected as -> " + user);
                // turn on auto commit because it's easy developement 
                // for this demo app
                connection.setAutoCommit(true);			
                // Create person table
    			String query = "CREATE TABLE person " + 
				"(id INTEGER IDENTITY PRIMARY KEY, " +
				"initials VARCHAR, " +
				"first_name VARCHAR, " +
				"middle_names VARCHAR, " +
				"last_name VARCHAR)";
    			CallableStatement statement = null;
    			try
				{
    		        statement = connection.prepareCall(query);
    		        statement.execute();
				}
    			catch (SQLException e1)
				{
    				log.error("Error during table setup!" + e1);
				}
    			finally
				{
    				try
					{
    					statement.close();
					}
    				catch (SQLException e2)
					{
    					log.error("Error closing statement: " + e2);
					}
				}
            }
        }
        catch (Exception e)
	    {
          log.error("Error during initialization: " +
          		e.getMessage());
	    }
	}
    
    /**
     * Creates a CallableStatement from the initialized connection
     * @param aParamSqlString is a query string with bind variables (they are written as '?')
     * @return a CallableStatement which parameters can be filled by the calling class
     */
    public static CallableStatement getCallableStatement(String aParamSqlString)
    {
        // just make sure you db connection is still alive
        createConnection();
      
        CallableStatement myStmnt = null;
        try
        {
            myStmnt = connection.prepareCall(aParamSqlString);
        }
        catch (SQLException e)
        {
            log.error("Error getting callable statement: " +
            	e.getMessage());
        }
        return myStmnt;
    }
}
